/*    A Simple Web Application

      Program file for server-side CGI execution
      This program is adapted from the Visual dBASE 7 
      Data Entry Web Wizard. 
      
      Author: Michael Nuwer (nuwermj@potsdam.edu)
      Date: 12/28/1999 
 
      Param:fOutFile is the CGI Response file name.
      It is created and opened by VdB_Cgi before calling.
      this application.


*/

Param fOutFile, cQuery
private aData, cStr, cID, cAction, cUsername
aData = new array()

Set talk OFF   ////// turn off interactive mode
Set century ON ////// Y2K fix 
cUsername = " "
cID = " "
cAction = " "

Try ////// Global try/catch prevents CGI lockup 
 
   // Get the Query string from the environment
   cEnv = getenv("QUERY_STRING")
   if len(trim(cEnv)) = 0
      errorPage("Message: No data received from server!",0)
   endif

   // EXEFILENAME is always the first pair
   // "&" is used as delimiter in CGI,
   //  so look for second pair
   if at("&",cEnv) > 0
      cEnv = substr(cEnv,at("&",cEnv)+1)
   endif
   do while at("=",cEnv) > 0
      // CGI uses "&" separators between pairs
      if at("&",cEnv) > 0 // if more than one pair left
         cData = OEMFormat(substr(cEnv,1,at("&",cEnv)-1))
         // Store the username to a memory variable. Below we will check for duplicate usernames
	 if at("username",cData) > 0   
            cUsername = trim(substr(cData,at("=",cData)+1))
         endif
	 // Store the ID to a memory variable.  
	 // Below this will be used to find the record that the web user wishes to modify.
	 if at("ID",cData) > 0
            cID = trim(substr(cData,at("=",cData)+1))
         endif

         cEnv = substr(cEnv,at("&",cEnv)+1)
      // If last pair 
      else
         cData = OEMFormat(cEnv)    // take the whole string
	 // Store the action to a memory variable.
	 // It is used in the Case Statement below to control the application.
	 // "action" will always be the last CGI pair.
	 if at("action",cData) > 0
	    cAction = trim(substr(cData,at("=",cData)+1))
         endif
         cEnv = " "
      endif
 
      if len(trim(cData)) > 0 .and. at("action",cData) = 0 .and. at("ID",cData) = 0
         // action and ID are passed from the web pages but are not saved in the data tables.  
	 // Thus these form fields are not added to the data array.
         aData.add(cData)
      endif
   enddo

   
   // The QUERY_STRING is now in an array called aData.  
   // The key field is stored in cID 
   // The web Form action is stored in cAction

   Select 1
   Use DATA.DBF Alias DATA
   Set Order to USERNAME
   // I do not want duplicate usernames in my tables.  
   // So I look up the Username here.
   Seek upper(cUsername)
   if Found()
      lOnFile = true
   else
      lOnFile = false
   endif   

   // Used for debugging
   *cMessage = "cAction = " + cAction 
   *GenerateMessage(fOutFile,cMessage)  
   
   // Okay, here we go.
   // cAction is passed from the web form and is used to control the dBASE application.  
   // The possible choices are:
   // Add -> this will generate a web page for data entry.
   // Add Record -> data passed from the data entry page is appended to the data table
   // Modify Record -> This will generate a web page that is populated with data from our table.
   // Modify This Record -> Data passed from the modify page replaces the existing data in the table.
   
   Do Case
   Case (ltrim(trim(cAction)) == "Add")
      
      print_add_page(fOutFile)

   Case (cAction == "Add Record")
      // Add the record passed from the add record page
      if lOnFile
        print_default(fOutFile, cUsername + " is already on file. Use the Modify Account Information link")
      else
        // add_record
	Append Blank

        // Sort array
        aData.Sort()
        cField = " "
        n=1
        Do while n <= alen(aData,1)
            ////// store array element for extraction
            ////// format: Field=Data
           cStr = trim(aData[n])
 
           if at("=",cStr) > 0  // if a valid name/value pair
               ////// extract field name
              cField = substr(cStr,1,at("=",cStr)-1)
               ////// extract data
              cData  = trim(substr(cStr,at("=",cStr)+1))
           endif
           replace &cField with cData
           ++n
        enddo
        cMessage="Record Added"
        print_default(fOutFile, cMessage)
      endif

   Case (cAction == "Modify Record")
      // Display the record in a web page.
      if lOnFile
         print_modify_page(fOutFile)
      else
         print_default(fOutFile, cUsername + " is not on file. Use the Add Account Information link. ")
      endif
	  
   Case (cAction == "Modify This Record")
      //  Modify the record that was passed.
      //
      //  The web form includes the value of ID so that we can find the record
      //  that is to be updated. ID is both unique and can not be modified by the user. 
      //  The web form has passed the updated information.  Now we will seek that record
      //  and replace the old information with the new information.
      
      Set Order to ID
      Seek Val(cID)
      if Found()

	  // Sort array
          aData.Sort()
          cField = " "
          n=1
          Do while n <= alen(aData,1)
              ////// store array element for extraction
              ////// format: Field=Data
              cStr = trim(aData[n])
 
              if at("=",cStr) > 0  // if a valid name/value pair
                  ////// extract field name
                 cField = substr(cStr,1,at("=",cStr)-1)
                  ////// extract data
                 cData  = trim(substr(cStr,at("=",cStr)+1))
               endif
               replace &cField with cData
     	       ++n
           enddo
           cMessage="Record Modified"
           print_default(fOutFile, cMessage)
      
      endif      
      
   Otherwise 
      print_default(fOutFile) 
   EndCase

catch (exception e)
 
    errorpage(e.message,e.lineno)
 
endtry 
 
Quit

//// Source: Visual dBASE 7 Web Wizard.
function ErrorPage(cMess,nLineNo)
   ////// Create new response file
   nLineNo = iif(empty(nLineNo),0,nLineNo)

   fOut = new file()
   fOut.Open(fOutFile,"RA")
   fOut.Puts("")

   ////// Generate HTML error page
   fOut.Puts(" ")
   fOut.Puts("<HTML>")
   fOut.Puts("<HEAD><Title>Visual dBASE 7 Web Wizard Error Page</Title>")
   fOut.Puts("<BODY>")
   fOut.Puts("")
   fOut.Puts("   <H2><I>Sorry,an error ocurred on the host!</I></H2>")
   fOut.Puts("")
   fOut.Puts("   <P>Error: "+cMess+" </P>")
   fOut.Puts("   <P>Line No.: "+ltrim(str(nLineNo,5))+"</p>")
   fOut.Puts("   <P>Press the back button and try again, or contact the Webmaster for assistance")
   fOut.Puts("")
   fOut.Puts("</BODY>")
   fOut.Puts("</HTML>")
   fOut.close()
   Quit
 
///////////// Function OEMFormat //////////////////////////////
///////////// Purpose - transforms CGI escape chars to OEM/////
///////////// Param: cStr - Environment Variable Value ////////
///////////// Source: Visual dBASE 7 Web Wizard.
function OEMFormat(cStr)
   ////// remove terminating character
   if at("%21",cStr) > 0
      cStr = substr(cStr,1,at("%21",cStr)-1)
   endif
   ////// remove "Pluses" used as space keepers in CGI
   do while at("+",cStr) > 0
      cStr = substr(cStr,1,at("+",cStr)-1) + " "+;
             substr(cStr,at("+",cStr)+1)
   enddo
   ////// add in "CRs" sent as escape characters
   do while at("%0D%0A",cStr) > 0
      cStr = substr(cStr,1,at("%0D%0A",cStr)-1) +chr(13)+;
             substr(cStr,at("%0D%0A",cStr)+3)
   enddo
   ////// convert escape characters back to oem chars
   nLeft = 1
   do while at("%",substr(cStr,nLeft) ) > 0
      nPos = at("%",substr(cStr,nLeft))+nLeft-1
      nLeft = nPos+1
      if nPos <= len(cStr)-2
         cStr = left(cStr,nPos-1)+ chr(htoi(substr(cStr,nPos+1,2)))+;
         substr(cStr,nPos+3)
      endif
   enddo
 
   return cStr



function GenerateMessage(fOutFile,cMessage)
   fOut = new File()
   If not file(fOutFile)
      fOut.Create(fOutFile,"RA")
   else
      fOut.Open(fOutFile,"RA")
   Endif
 
   fOut.Puts(" ")
   fOut.Puts("<HTML>")
   fOut.Puts("<TITLE>Data Entered</TITLE>")
   fOut.Puts('<body bgcolor="#C0C0C0" background="None">')
   fOut.Puts("<blockquote>")
   fOut.Puts(cMessage)
   fOut.Puts("</Blockquote>")
   fOut.Puts("</Body>")
   fOut.Puts("</HTML>")
 
   fOut.close()
   quit


Function print_modify_page(fOutFile)

   fOut = new file()
   fOut.Open(fOutFile,"RA")
   fOut.Puts("")

   fOut.Puts([<HTML>])
   fOut.Puts([<HEAD>])
   fOut.Puts([   <META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=iso-8859-1">])
   fOut.Puts([   <TITLE>Modify Record</TITLE>])
   fOut.Puts([<SCRIPT LANGUAGE=JAVASCRIPT>])
   fOut.Puts([   function helpWindow() {])
   fOut.Puts([      newWindow = window.open('help.html','helpWin', 'width=250,height=400')])
   fOut.Puts([      newWindow.focus() ])
   fOut.Puts([}])
   fOut.Puts([</SCRIPT>])

   fOut.Puts([</HEAD>])
   fOut.Puts([<BODY BGCOLOR="#FFFFFF"><BR>])

   fOut.Puts([<!-- Table One the main outer -->])
   fOut.Puts([<table align="center" border="0" cellspacing="0" cellpadding="3" bgcolor="#151b8d" width=460>])
   fOut.Puts([<tr><td align="center" colspan=2 valign=top>])
   fOut.Puts([<!-- Table Two -->])
   fOut.Puts([<table height="" cellspacing="0" cellpadding="3" border="0" width=460>])
   fOut.Puts([<tr><!-- Row one --><td align="center" bgcolor="lightyellow">])      


   fOut.Puts([<BR><CENTER><B><FONT FACE="ARIAL"><FONT SIZE=+1>Modify this Record</FONT></FONT></B></CENTER>])


   fOut.Puts([<P><Form METHOD="GET" ACTION="/bulletin/VdB_CGI.exe">])
   fOut.Puts([<INPUT TYPE="hidden" NAME="EXEFILENAME" VALUE=dbaseapp.EXE>])
   fOut.Puts([<INPUT TYPE="hidden" NAME="ID" VALUE="] + str(data->ID,3,0) + [">])

   fOut.Puts([<CENTER><TABLE BORDER=0 BGCOLOR="lightyellow" CELLSPACING=0 >])
   fOut.Puts([<TR>])
   fOut.Puts([<TD><B>Username:</B></TD>])

   fOut.Puts([<TD><INPUT TYPE=TEXT NAME="username"])
   If not empty(data->username)
      fOut.Puts([VALUE="] + trim(data->username) + ["])
   Endif
   fOut.Puts([></TD>])
   fOut.Puts([</TR>])

   fOut.Puts([<TR>])
   fOut.Puts([<TD><B>SSN:</B></TD>])

   fOut.Puts([<TD><INPUT TYPE=TEXT NAME="SSN"])
   If not empty(data->ssn)
      fOut.Puts([VALUE="] + trim(data->SSN) + ["])
   Endif
   fOut.Puts([></TD>])

   fOut.Puts([</TR>])

   fOut.Puts([<TR>])
   fOut.Puts([<TD><B>Last Name:</B></TD>])


   fOut.Puts([<TD><INPUT TYPE=TEXT NAME="Lname"])
   If not empty(data->lname)
      fOut.Puts([VALUE="] + trim(data->lname) + ["])
   Endif
   fOut.Puts([></TD>])
   fOut.Puts([</TR>])
   fOut.Puts([<TR>])
   fOut.Puts([<TD><B>First Name:</B></TD>])

   fOut.Puts([<TD><INPUT TYPE=TEXT NAME="Fname"])

   If not empty(data->Fname)
      fOut.Puts([VALUE="] + trim(data->Fname) + ["])
   Endif
   fOut.Puts([></TD>])
   fOut.Puts([</TR>])
   fOut.Puts([<TR>])
   fOut.Puts([<TD><B>Email:</B></TD>])

   fOut.Puts([<TD><INPUT TYPE=TEXT NAME="email"])
   If not empty(data->email)
      fOut.Puts([VALUE="] + trim(data->email) + ["])
   Endif
   fOut.Puts([></TD>])
   fOut.Puts([</TR>])

   fOut.Puts([<TR>])
   fOut.Puts([<TD COLSPAN="2">])
   fOut.Puts([<CENTER><BR><INPUT TYPE=SUBMIT NAME=action VALUE="Modify This Record"><INPUT TYPE=SUBMIT NAME=action VALUE="Abandon Changes"></CENTER>])
   fOut.Puts([<CENTER></CENTER>])
   fOut.Puts([</TD>])
   fOut.Puts([</TR>])
   fOut.Puts([</TABLE></CENTER><BR>])

   fOut.Puts([<!-- End Table Two --></td></tr></table>])
   fOut.Puts([<!-- End Table one --></td></tr></table>])
   fOut.Puts([<BR><CENTER>Click <A HREF="javascript:helpWindow()">Here</A> for Help</CENTER>])


   fOut.Puts([</BODY>])
   fOut.Puts([</HTML>])

   fOut.close()
   Quit

function print_add_page(fOutFile)
   fOut = new file()
   fOut.Open(fOutFile,"RA")
   fOut.Puts("")


   fOut.Puts([<HTML>])
   fOut.Puts([<HEAD>])
   fOut.Puts([   <META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=iso-8859-1">])
   fOut.Puts([   <TITLE>Add a Record</TITLE>])
   fOut.Puts([<SCRIPT LANGUAGE=JAVASCRIPT>])
   fOut.Puts([   function helpWindow() {])
   fOut.Puts([      newWindow = window.open('help.html','helpWin', 'width=250,height=400')])
   fOut.Puts([      newWindow.focus() ])
   fOut.Puts([}])
   fOut.Puts([</SCRIPT>])


   fOut.Puts([</HEAD>])
   fOut.Puts([<BODY BGCOLOR="#FFFFFF">])
   
   fOut.Puts([<!-- Table One the main outer -->])
   fOut.Puts([<table align="center" border="0" cellspacing="0" cellpadding="3" bgcolor="#151b8d" width=460>])
   fOut.Puts([<tr><td align="center" colspan=2 valign=top>])
   fOut.Puts([<!-- Table Two -->])
   fOut.Puts([<table height="" cellspacing="0" cellpadding="3" border="0" width=460>])
   fOut.Puts([<tr><!-- Row one --><td align="center" bgcolor="lightyellow">])      

   
   fOut.Puts([<BR><CENTER><B><FONT FACE="ARIAL" SIZE=+1>Add a Record</FONT></B></CENTER>])

   fOut.Puts([<P><Form METHOD="GET" ACTION="/bulletin/VdB_CGI.exe">])
   fOut.Puts([<input TYPE="hidden" NAME="EXEFILENAME" VALUE=dbaseapp.EXE>])

   fOut.Puts([<CENTER><TABLE BGCOLOR="lightyellow" BORDER=0 CELLSPACING=0 >])
   fOut.Puts([<TR>])
   fOut.Puts([<TD><B>Username:</B></TD>])

   fOut.Puts([<TD><INPUT TYPE=TEXT NAME="username"></TD>])
   fOut.Puts([</TR>])

   fOut.Puts([<TR>])
   fOut.Puts([<TD><B>SSN:</B></TD>])

   fOut.Puts([<TD><INPUT TYPE=TEXT NAME="SSN"></TD>])
   fOut.Puts([</TR>])

   fOut.Puts([<TR>])
   fOut.Puts([<TD><B>Last Name:</B></TD>])

   fOut.Puts([<TD><INPUT TYPE=TEXT NAME="Lname"></TD>])
   fOut.Puts([</TR>])
   fOut.Puts([<TR>])
   fOut.Puts([<TD><B>First Name:</B></TD>])

   fOut.Puts([<TD><INPUT TYPE=TEXT NAME="Fname"></TD>])
   fOut.Puts([</TR>])
   fOut.Puts([<TR>])
   fOut.Puts([<TD><B>Email:</B></TD>])

   fOut.Puts([<TD BGCOLOR="lightyellow"><INPUT TYPE=TEXT NAME="email"></TD>])
   fOut.Puts([</TR>])

   fOut.Puts([<TR>])
   fOut.Puts([<TD COLSPAN="2">])
   fOut.Puts([<CENTER><BR><INPUT TYPE=SUBMIT NAME=action VALUE="Add Record"><INPUT TYPE=SUBMIT NAME=action VALUE="Abandon Changes"></CENTER>])
   fOut.Puts([</TD>])
   fOut.Puts([</TR>])
   fOut.Puts([</TABLE></CENTER><BR>])

   fOut.Puts([<!-- End Table Two --></td></tr></table>])
   fOut.Puts([<!-- End Table one --></td></tr></table>])

   fOut.Puts([<BR><CENTER>Click <A HREF="javascript:helpWindow()">Here</A> for Help</CENTER>])

   fOut.Puts([</BODY>])
   fOut.Puts([</HTML>])

   fOut.close()
   Quit


function print_default(fOutFile,cMessage)

   fOut = new file()
   fOut.Open(fOutFile,"RA")
   fOut.Puts("")

   fOut.Puts([<HTML>])
   fOut.Puts([<HEAD>])
   fOut.Puts([   <META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=iso-8859-1">])
   fOut.Puts([   <TITLE>Simple Database Main Screen</TITLE>])
   
   fOut.Puts([<SCRIPT LANGUAGE=JAVASCRIPT>])
   fOut.Puts([   function helpWindow() {])
   fOut.Puts([      newWindow = window.open('help.html','helpWin', 'width=250,height=400')])
   fOut.Puts([      newWindow.focus() ])
   fOut.Puts([}])
   fOut.Puts([</SCRIPT>])

   
   
   fOut.Puts([</HEAD>])
   fOut.Puts([<BODY BGCOLOR="#FFFFFF"><BR>])

   fOut.Puts([<!-- Table One the main outer -->])
   fOut.Puts([<table align="center" border="0" cellspacing="0" cellpadding="3" bgcolor="#151b8d" width=460>])
   fOut.Puts([<tr><td align="center" colspan=2 valign=top>])
   fOut.Puts([<!-- Table Two -->])
   fOut.Puts([<table height="" cellspacing="0" cellpadding="3" border="0" width=460>])
   fOut.Puts([<tr><!-- Row one --><td align="center" bgcolor="lightyellow">])      


   if not empty(cMessage)
      fOut.Puts([<CENTER><B><FONT FACE="ARIAL" SIZE=+1>] + cMessage + [</FONT></B></CENTER>])
   else
      fOut.Puts([<CENTER><B><FONT FACE="ARIAL" SIZE=+1>A Simple Database</FONT></B></CENTER>])
   endif 
   fOut.Puts([<BR>])
   fOut.Puts([<FORM METHOD="GET" ACTION="/bulletin/VdB_CGI.exe">])
   fOut.Puts([<input TYPE="hidden" NAME="EXEFILENAME" VALUE=dbaseapp.EXE>])


   fOut.Puts([<CENTER><TABLE BORDER=0 CELLSPACING=0 WIDTH="50%" BGCOLOR="lightyellow" >])
   fOut.Puts([<TR><TD COLSPAN="2">])

   fOut.Puts([<FONT FACE="ARIAL" SIZE=-1>To <I>add</I> a record, click])
   fOut.Puts([on the Add button.</FONT>  ])

   fOut.Puts([<FONT FACE="ARIAL" SIZE=-1>To <I>modify</I> records, enter])
   fOut.Puts([your Username in the box below then click the modify button.</FONT>])
   fOut.Puts([</TD></TR>])

   fOut.Puts([<TR><TD COLSPAN="2">])
   fOut.Puts([<BR><CENTER><B><FONT FACE="ARIAL" SIZE=-1>User to Modify&nbsp;</FONT></CENTER></B></TD></TR>])
   fOut.Puts([<TR><TD><B><FONT FACE="ARIAL" SIZE=-1>Username:&nbsp;</FONT></B></TD>])

   fOut.Puts([<TD><INPUT TYPE="text" NAME="username" SIZE="40"></TD>])
   fOut.Puts([</TR>])

   fOut.Puts([<TR>])
   fOut.Puts([<TD COLSPAN="2">])
   fOut.Puts([<CENTER><BR>])
   fOut.Puts([<INPUT TYPE="submit" NAME="action" VALUE="   Add   ">])
   fOut.Puts([<INPUT TYPE="submit" NAME="action" VALUE="Modify Record">])
   fOut.Puts([</CENTER>])
   fOut.Puts([</TD>])
   fOut.Puts([</TR>])
   fOut.Puts([</TABLE></CENTER></FORM>])

   fOut.Puts([<!-- End Table Two --></td></tr></table>])
   fOut.Puts([<!-- End Table one --></td></tr></table>])

   fOut.Puts([<BR><CENTER>Click <A HREF="javascript:helpWindow()">Here</A> for Help</CENTER>])
   fOut.Puts([<BR><CENTER><FONT FACE="ARIAL" SIZE=-1>You are welcome to test this application by Adding a name <BR> and then Modify that record.  The data is indexed by username.</FONT></CENTER>])
   fOut.Puts([<BR><CENTER><IMG SRC="PoweredBy.gif" BORDER=0> </CENTER> </BODY>])

   fOut.Puts([</HTML>])
   fOut.close()
   Quit


